@@ -1,16 +1,22 @@
+from httptools import HttpParserUpgrade
+from websockets import ConnectionClosed  # noqa
+from websockets import InvalidHandshake, WebSocketCommonProtocol, handshake
+
 from sanic.exceptions import InvalidUsage
 from sanic.server import HttpProtocol
-from httptools import HttpParserUpgrade
-from websockets import handshake, WebSocketCommonProtocol, InvalidHandshake
-from websockets import ConnectionClosed  # noqa
 
 
 class WebSocketProtocol(HttpProtocol):
-    def __init__(self, *args, websocket_timeout=10,
-                 websocket_max_size=None,
-                 websocket_max_queue=None,
-                 websocket_read_limit=2 ** 16,
-                 websocket_write_limit=2 ** 16, **kwargs):
+    def __init__(
+        self,
+        *args,
+        websocket_timeout=10,
+        websocket_max_size=None,
+        websocket_max_queue=None,
+        websocket_read_limit=2 ** 16,
+        websocket_write_limit=2 ** 16,
+        **kwargs
+    ):
         super().__init__(*args, **kwargs)
         self.websocket = None
         self.websocket_timeout = websocket_timeout
@@ -57,36 +63,32 @@ class WebSocketProtocol(HttpProtocol):
 
     async def websocket_handshake(self, request, subprotocols=None):
         # let the websockets package do the handshake with the client
-        headers = []
-
-        def get_header(k):
-            return request.headers.get(k, '')
-
-        def set_header(k, v):
-            headers.append((k, v))
+        headers = {}
 
         try:
-            key = handshake.check_request(get_header)
-            handshake.build_response(set_header, key)
+            key = handshake.check_request(request.headers)
+            handshake.build_response(headers, key)
         except InvalidHandshake:
-            raise InvalidUsage('Invalid websocket request')
+            raise InvalidUsage("Invalid websocket request")
 
         subprotocol = None
-        if subprotocols and 'Sec-Websocket-Protocol' in request.headers:
+        if subprotocols and "Sec-Websocket-Protocol" in request.headers:
             # select a subprotocol
-            client_subprotocols = [p.strip() for p in request.headers[
-                'Sec-Websocket-Protocol'].split(',')]
+            client_subprotocols = [
+                p.strip()
+                for p in request.headers["Sec-Websocket-Protocol"].split(",")
+            ]
             for p in client_subprotocols:
                 if p in subprotocols:
                     subprotocol = p
-                    set_header('Sec-Websocket-Protocol', subprotocol)
+                    headers["Sec-Websocket-Protocol"] = subprotocol
                     break
 
         # write the 101 response back to the client
-        rv = b'HTTP/1.1 101 Switching Protocols\r\n'
-        for k, v in headers:
-            rv += k.encode('utf-8') + b': ' + v.encode('utf-8') + b'\r\n'
-        rv += b'\r\n'
+        rv = b"HTTP/1.1 101 Switching Protocols\r\n"
+        for k, v in headers.items():
+            rv += k.encode("utf-8") + b": " + v.encode("utf-8") + b"\r\n"
+        rv += b"\r\n"
         request.transport.write(rv)
 
         # hook up the websocket protocol
@@ -95,7 +97,7 @@ class WebSocketProtocol(HttpProtocol):
             max_size=self.websocket_max_size,
             max_queue=self.websocket_max_queue,
             read_limit=self.websocket_read_limit,
-            write_limit=self.websocket_write_limit
+            write_limit=self.websocket_write_limit,
         )
         self.websocket.subprotocol = subprotocol
         self.websocket.connection_made(request.transport)
